import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { useState, useEffect, useMemo } from "react";
import { createPortal } from "react-dom";
import { useDispatch } from "react-redux";
import { Box } from "@primer/react";
import { useJupyter } from "./../../jupyter/JupyterContext";
import { Kernel } from "./../../jupyter/services/kernel/Kernel";
import Lumino from '../../jupyter/lumino/LuminoNotebook';
import { asObservable } from './../../jupyter/lumino/LuminoObservable';
import CellMetadataEditor from './cell/metadata/CellMetadataEditor';
import NotebookAdapter from './NotebookAdapter';
import { notebookActions, selectNotebookPortals, notebookEpics, notebookReducer } from './NotebookState';
import '@jupyterlab/completer/style/index.css';
import '@jupyterlab/documentsearch/style/index.css';
import '@jupyterlab/codemirror/style/index.css';
import '@jupyterlab/notebook/style/index.css';
import '@jupyterlab/theme-light-extension/style/theme.css';
import '@jupyterlab/theme-light-extension/style/variables.css';
import './Notebook.css';
const LuminoNotebook = (props) => {
    const { adapter } = props;
    return _jsx(Lumino, { adapter: adapter }, void 0);
};
/**
 * This component creates a Notebook as a collection of snippets
 * with sidebars.
 *
 * @param props The notebook properties.
 * @returns A Notebook React.js component.
 */
export const Notebook = (props) => {
    const { serviceManager, kernelManager, injectableStore } = useJupyter();
    const { readOnly, cellMetadataPanel, nbgrader, uid } = props;
    const dispatch = useDispatch();
    const portals = selectNotebookPortals();
    const [adapter, setAdapter] = useState();
    const [_kernel, setKernel] = useState();
    useMemo(() => {
        injectableStore.inject('notebook', notebookReducer, notebookEpics);
    }, []);
    useEffect(() => {
        if (serviceManager && kernelManager) {
            const kernel = readOnly ? undefined : new Kernel({ kernelManager });
            const adapter = new NotebookAdapter(Object.assign(Object.assign({}, props), { kernel }), injectableStore, serviceManager);
            setKernel(kernel);
            setAdapter(adapter);
            dispatch(notebookActions.update({ adapter }));
            adapter.serviceManager.ready.then(() => {
                var _a, _b, _c, _d, _e;
                adapter.loadNotebook();
                if (!readOnly && cellMetadataPanel) {
                    const activeCellChanged$ = asObservable(adapter.notebookPanel.content.activeCellChanged);
                    activeCellChanged$.subscribe((activeCellChanged) => {
                        dispatch(notebookActions.activeCellChange(activeCellChanged));
                        const panelDiv = document.getElementById('right-panel-id');
                        const cellMetadataOptions = (_jsx(Box, Object.assign({ mt: 3 }, { children: _jsx(CellMetadataEditor, { cell: activeCellChanged, nbgrader: nbgrader }, void 0) }), void 0));
                        const portal = createPortal(cellMetadataOptions, panelDiv);
                        dispatch(notebookActions.setPortal({ portal, pinned: false }));
                    });
                }
                (_b = (_a = adapter.notebookPanel) === null || _a === void 0 ? void 0 : _a.model) === null || _b === void 0 ? void 0 : _b.contentChanged.connect((notebookModel, _) => {
                    dispatch(notebookActions.modelChange(notebookModel));
                });
                (_c = adapter.notebookPanel) === null || _c === void 0 ? void 0 : _c.content.activeCellChanged.connect((_, activeCellChanged) => {
                    dispatch(notebookActions.activeCellChange(activeCellChanged));
                });
                (_d = adapter.notebookPanel) === null || _d === void 0 ? void 0 : _d.model.sharedModel.changed.connect((_, notebookChange) => {
                    dispatch(notebookActions.notebookChange(notebookChange));
                });
                (_e = adapter.notebookPanel) === null || _e === void 0 ? void 0 : _e.sessionContext.statusChanged.connect((_, kernelStatusChanged) => {
                    dispatch(notebookActions.kernelStatusChanged(kernelStatusChanged));
                });
            });
            return () => {
                setAdapter(undefined);
                setKernel(undefined);
                dispatch(notebookActions.setPortal(undefined));
            };
        }
    }, [uid, serviceManager, kernelManager]);
    return (_jsx(Box, { children: _jsxs(Box, Object.assign({ css: {
                '& .dla-JupyterLab-Notebook': {
                    height: 'calc(100vh - 220px)',
                    overflowY: 'hidden',
                },
                '& .jp-Notebook': {
                    flex: '1 1 auto !important',
                },
                '& .jp-NotebookPanel': {
                //            borderBottom: '1px solid #e0e0e0',
                },
                '& .jp-Toolbar': {
                    //            display: readOnly ? 'none' : 'flex',
                    display: 'none',
                    zIndex: 0,
                },
                '& .jp-Toolbar .jp-HTMLSelect.jp-DefaultStyle select': {
                    fontSize: '14px',
                },
                '& .jp-Toolbar > .jp-Toolbar-responsive-opener': {
                    display: 'none',
                },
                '& .jp-Toolbar-kernelName': {
                    display: 'none',
                },
                '& .jp-Cell': {
                    width: `calc(100% - ${props.cellSidebarMargin}px)`,
                },
                '& .jp-Cell .jp-CellHeader': {
                    position: 'absolute',
                    top: '-5px',
                    left: `${props.cellSidebarMargin + 10}px`,
                    height: 'auto',
                },
                '& .jp-Cell .dla-CellHeader-Container': {
                    padding: '4px 8px',
                    width: `${props.cellSidebarMargin + 10}px`,
                    marginLeft: 'auto',
                },
            } }, { children: [_jsx(_Fragment, { children: portals.map((portal) => portal) }, void 0),
                _jsx(Box, { children: adapter &&
                        _jsx(LuminoNotebook, { adapter: adapter }, void 0) }, void 0)] }), void 0) }, void 0));
};
Notebook.defaultProps = {
    ipywidgets: 'lab',
    readOnly: false,
    nbgrader: false,
    cellMetadataPanel: true,
    cellSidebarMargin: 120,
};
export default Notebook;
//# sourceMappingURL=Notebook.js.map