import { CommandRegistry } from '@lumino/commands';
import { BoxPanel, Widget } from '@lumino/widgets';
import { PageConfig } from '@jupyterlab/coreutils';
import { CodeCellModel, MarkdownCellModel, RawCellModel } from '@jupyterlab/cells';
import { DocumentRegistry, Context } from '@jupyterlab/docregistry';
import { standardRendererFactories, RenderMimeRegistry } from '@jupyterlab/rendermime';
import { rendererFactory as jsonRendererFactory } from '@jupyterlab/json-extension';
import { rendererFactory as javascriptRendererFactory } from '@jupyterlab/javascript-extension';
import { NotebookPanel, NotebookWidgetFactory, NotebookTracker, NotebookActions } from '@jupyterlab/notebook';
import { CodeMirrorEditorFactory, CodeMirrorMimeTypeService } from '@jupyterlab/codemirror';
import { Completer, CompleterModel, CompletionHandler, ConnectorProxy, KernelCompleterProvider } from '@jupyterlab/completer';
import { MathJaxTypesetter } from '@jupyterlab/mathjax2';
import { newUuid } from './../../jupyter/utils/Ids';
import getMarked from './marked/marked';
import { requireLoader } from "@jupyter-widgets/html-manager";
import { WIDGET_MIMETYPE, WidgetRenderer } from "@jupyter-widgets/html-manager/lib/output_renderers";
import CustomNotebookModelFactory from './model/CustomNotebookModelFactory';
import { NotebookCommands } from './NotebookCommands';
import CellSidebarContentFactory from './cell/sidebar/base/CellSidebarContentFactory';
import { IPyWidgetsClassicManager } from "./../../jupyter/ipywidgets/IPyWidgetsClassicManager";
import { activateWidgetExtension } from "./../../jupyter/ipywidgets/IPyWidgetsJupyterLabPlugin";
import { activatePlotlyWidgetExtension } from "./../../jupyter/ipywidgets/plotly/JupyterlabPlugin";
export class NotebookAdapter {
    constructor(props, store, serviceManager) {
        this.setDefaultCellType = (cellType) => {
            this._notebookPanel.content.notebookConfig.defaultCell = cellType;
        };
        this.changeCellType = (cellType) => {
            var _a;
            //    NotebookActions.changeCellType(this._notebookPanel?.content!, cellType);
            this.doChangeCellType((_a = this._notebookPanel) === null || _a === void 0 ? void 0 : _a.content, cellType);
        };
        this.dispose = () => {
            var _a, _b;
            (_a = this._context) === null || _a === void 0 ? void 0 : _a.dispose();
            (_b = this._notebookPanel) === null || _b === void 0 ? void 0 : _b.dispose();
            //    this.kernel?.getJupyterKernel().then(k => k.shutdown());
        };
        this._props = props;
        this._store = store;
        this._serviceManager = serviceManager;
        this._model = props.model;
        this._CellSidebar = props.CellSidebar;
        this._nbgrader = props.nbgrader;
        this._readOnly = props.readOnly;
        this._ipywidgets = props.ipywidgets;
        this._kernel = props.kernel;
        this._boxPanel = new BoxPanel();
        this._boxPanel.addClass('dla-JupyterLab-Notebook');
        this._boxPanel.spacing = 0;
        this._uid = props.uid;
        this._commandRegistry = new CommandRegistry();
        this.loadNotebook = this.loadNotebook.bind(this);
    }
    loadNotebook() {
        var _a;
        const useCapture = true;
        document.addEventListener('keydown', event => { var _a; (_a = this._commandRegistry) === null || _a === void 0 ? void 0 : _a.processKeydownEvent(event); }, useCapture);
        const rendererFactories = standardRendererFactories.filter(factory => factory.mimeTypes[0] !== 'text/javascript');
        rendererFactories.push(jsonRendererFactory);
        rendererFactories.push(javascriptRendererFactory);
        const rendermime = new RenderMimeRegistry({
            initialFactories: rendererFactories,
            latexTypesetter: new MathJaxTypesetter({
                url: PageConfig.getOption('mathjaxUrl'),
                config: PageConfig.getOption('mathjaxConfig'),
            }),
            markdownParser: getMarked(),
        });
        const documentRegistry = new DocumentRegistry({});
        const mimeTypeService = new CodeMirrorMimeTypeService();
        const editorServices = {
            factoryService: new CodeMirrorEditorFactory(),
            mimeTypeService,
        };
        const editorFactory = editorServices.factoryService.newInlineEditor;
        const contentFactory = this._CellSidebar
            ?
                new CellSidebarContentFactory(this._CellSidebar, this._nbgrader, this._commandRegistry, this._store, { editorFactory })
            :
                new NotebookPanel.ContentFactory({ editorFactory });
        this._tracker = new NotebookTracker({ namespace: this._uid });
        if (this._ipywidgets === 'classic') {
            this._iPyWidgetsClassicManager = new IPyWidgetsClassicManager({ loader: requireLoader });
            rendermime.addFactory({
                safe: false,
                mimeTypes: [WIDGET_MIMETYPE],
                createRenderer: (options) => new WidgetRenderer(options, this._iPyWidgetsClassicManager),
            }, 1);
        }
        else if (this._ipywidgets === 'lab') {
            const widgetRegistry = activateWidgetExtension(rendermime, this._tracker, null, null);
            activatePlotlyWidgetExtension(widgetRegistry);
        }
        const notebookWidgetFactory = new NotebookWidgetFactory({
            name: 'Notebook',
            modelName: 'notebook',
            fileTypes: ['notebook'],
            defaultFor: ['notebook'],
            preferKernel: true,
            canStartKernel: false,
            rendermime,
            contentFactory,
            mimeTypeService: editorServices.mimeTypeService,
        });
        /*
        notebookWidgetFactory.widgetCreated.connect((sender, widget) => {
          widget.context.pathChanged.connect(() => {
            void this._tracker?.save(widget);
          });
          void this._tracker?.add(widget);
        });
        */
        documentRegistry.addWidgetFactory(notebookWidgetFactory);
        const notebookModelFactory = new CustomNotebookModelFactory({});
        documentRegistry.addModelFactory(notebookModelFactory);
        this._context = new Context({
            manager: this._serviceManager,
            factory: notebookModelFactory,
            path: this._props.path || "ping.ipynb",
            kernelPreference: {
                shouldStart: false,
                autoStartDefault: false,
                shutdownOnDispose: true,
                //        id: this.kernel?.kernelId,
                //        name: 'python3',
            }
        });
        const setupCompleter = (notebookPanel) => {
            const editor = notebookPanel.content.activeCell && notebookPanel.content.activeCell.editor;
            const sessionContext = notebookPanel.context.sessionContext;
            const completerModel = new CompleterModel();
            const completer = new Completer({ editor, model: completerModel });
            const completerTimeout = 1000;
            const provider = new KernelCompleterProvider();
            const connector = new ConnectorProxy({
                widget: notebookPanel,
                editor,
                session: sessionContext.session
            }, [provider], completerTimeout);
            const handler = new CompletionHandler({ completer, connector });
            void sessionContext.ready.then(() => {
                const provider = new KernelCompleterProvider();
                const connector = new ConnectorProxy({ widget: this._notebookPanel, editor, session: sessionContext.session }, [provider], completerTimeout);
                handler.connector = connector;
            });
            handler.editor = editor;
            notebookPanel.content.activeCellChanged.connect((sender, snippet) => {
                handler.editor = snippet && snippet.editor;
            });
            completer.hide();
            Widget.attach(completer, document.body);
            return handler;
        };
        this._notebookPanel = (_a = documentRegistry.getWidgetFactory('notebook')) === null || _a === void 0 ? void 0 : _a.createNew(this._context);
        const isNew = (this._props.path !== "") ? false : true;
        this._context.initialize(isNew).then(() => {
            var _a, _b, _c;
            if (this._kernel) {
                (_a = this._kernel) === null || _a === void 0 ? void 0 : _a.getJupyterKernel().then((kernel) => {
                    var _a;
                    //          if (this.context?.sessionContext.hasNoKernel) {
                    (_a = this._context) === null || _a === void 0 ? void 0 : _a.sessionContext.changeKernel(kernel.model).then(() => {
                        const completerHandler = setupCompleter(this._notebookPanel);
                        NotebookCommands(this._commandRegistry, this._notebookPanel, completerHandler, this._props);
                    });
                    //          }
                });
            }
            if (this._model) {
                (_c = (_b = this._notebookPanel) === null || _b === void 0 ? void 0 : _b.model) === null || _c === void 0 ? void 0 : _c.cells.clear();
                this._model.cells.map((cell, index) => {
                    var _a, _b, _c, _d, _e, _f;
                    switch (cell.cell_type) {
                        case "code": {
                            const cellModel = new CodeCellModel({
                                cell: Object.assign(Object.assign({}, cell), { metadata: Object.assign(Object.assign({}, cell.metadata), { editable: !this._readOnly }) }),
                                id: cell.id ? cell.id.toString() : newUuid(),
                            });
                            (_b = (_a = this._notebookPanel) === null || _a === void 0 ? void 0 : _a.model) === null || _b === void 0 ? void 0 : _b.cells.push(cellModel);
                            break;
                        }
                        case "markdown": {
                            const cellModel = new MarkdownCellModel({
                                cell: Object.assign(Object.assign({}, cell), { metadata: Object.assign(Object.assign({}, cell.metadata), { editable: !this._readOnly }) }),
                                id: cell.id ? cell.id.toString() : newUuid(),
                            });
                            (_d = (_c = this._notebookPanel) === null || _c === void 0 ? void 0 : _c.model) === null || _d === void 0 ? void 0 : _d.cells.push(cellModel);
                            break;
                        }
                        case "raw": {
                            const cellModel = new RawCellModel({
                                cell: Object.assign(Object.assign({}, cell), { metadata: Object.assign(Object.assign({}, cell.metadata), { editable: !this._readOnly }) }),
                                id: cell.id ? cell.id.toString() : newUuid(),
                            });
                            (_f = (_e = this._notebookPanel) === null || _e === void 0 ? void 0 : _e.model) === null || _f === void 0 ? void 0 : _f.cells.push(cellModel);
                            break;
                        }
                    }
                });
            }
        });
        if (this._ipywidgets === 'classic') {
            this._notebookPanel.sessionContext.kernelChanged.connect((sender, args) => {
                var _a;
                (_a = this._iPyWidgetsClassicManager) === null || _a === void 0 ? void 0 : _a.registerWithKernel(args.newValue);
            });
        }
        BoxPanel.setStretch(this._notebookPanel, 0);
        this._boxPanel.addWidget(this._notebookPanel);
        window.addEventListener('resize', () => {
            var _a;
            (_a = this._notebookPanel) === null || _a === void 0 ? void 0 : _a.update();
        });
        const tracker = this._tracker;
        function getCurrent(args) {
            const widget = tracker.currentWidget;
            return widget;
        }
        function isEnabled() {
            return (tracker.currentWidget !== null);
        }
        this._commandRegistry.addCommand('run-selected-codecell', {
            label: 'Run Cell',
            execute: args => {
                const current = getCurrent(args);
                if (current) {
                    const { context, content } = current;
                    NotebookActions.run(content, context.sessionContext);
                }
            },
            isEnabled,
        });
    }
    get uid() {
        return this._uid;
    }
    get notebookPanel() {
        return this._notebookPanel;
    }
    get commands() {
        return this._commandRegistry;
    }
    get panel() {
        return this._boxPanel;
    }
    get serviceManager() {
        return this._serviceManager;
    }
    doChangeCellType(notebook, value) {
        const model = notebook.model;
        const cells = model.cells;
        cells.beginCompoundOperation();
        notebook.widgets.forEach((child, index) => {
            if (!notebook.isSelectedOrActive(child)) {
                return;
            }
            if (child.model.type !== value) {
                const cell = child.model.toJSON();
                let newCell;
                switch (value) {
                    case 'code':
                        newCell = model.contentFactory.createCodeCell({ cell });
                        break;
                    case 'markdown':
                        newCell = model.contentFactory.createMarkdownCell({ cell });
                        if (child.model.type === 'code') {
                            newCell.trusted = false;
                        }
                        break;
                    default:
                        newCell = model.contentFactory.createRawCell({ cell });
                        if (child.model.type === 'code') {
                            newCell.trusted = false;
                        }
                }
                cells.set(index, newCell);
            }
            if (value === 'markdown') {
                // Fetch the new widget and unrender it.
                child = notebook.widgets[index];
                child.rendered = false;
            }
        });
        cells.endCompoundOperation();
        notebook.deselectAll();
    }
}
export default NotebookAdapter;
//# sourceMappingURL=NotebookAdapter.js.map