import { jsx as _jsx } from "@emotion/react/jsx-runtime";
import { useState, useEffect, useContext, createContext } from "react";
import { Provider as ReduxProvider } from "react-redux";
import { ServiceManager, ServerConnection } from '@jupyterlab/services';
import { getJupyterServerHttpUrl } from './JupyterConfig';
import { requestAPI } from './JupyterHandlers';
import { startLiteServer } from './../jupyter/lite/LiteServer';
import Kernel from './services/kernel/Kernel';
/**
 * The instance for the Jupyter context.
 */
export const JupyterContext = createContext(undefined);
export const useJupyter = () => {
    const context = useContext(JupyterContext);
    if (!context)
        throw new Error("useContext must be inside a provider with a value.");
    return context;
};
/**
 * The type for the Jupyter context consumer.
 */
export const JupyterContextConsumer = JupyterContext.Consumer;
/**
 * The type for the Jupyter context provider.
 */
const JupyterProvider = JupyterContext.Provider;
/**
 * Utiliy method to ensure the Jupyter context is authenticated
 * with the Jupyter server.
 */
export const ensureJupyterAuth = (serverSettings) => {
    return requestAPI(serverSettings, 'api', '').then(data => {
        return true;
    })
        .catch(reason => {
        console.log('The jupyter server API has failed with reason', reason);
        return false;
    });
};
const headers = new Headers({
    "Cache-Control": "no-cache, no-store, must-revalidate",
    "Pragma": "no-cache",
    "Expires": "0",
});
/**
 * The Jupyter context provider.
 */
export const JupyterContextProvider = ({ children, lite, createKernel, variant, baseUrl, wsUrl, injectableStore }) => {
    const [_, setVariant] = useState('default');
    const [serverSettings] = useState(ServerConnection.makeSettings({
        baseUrl,
        wsUrl,
        appendToken: true,
        init: {
            credentials: "include",
            mode: 'cors',
            headers,
        }
    }));
    const [serviceManager, setServiceManager] = useState();
    const [kernelManager, setKernelManager] = useState();
    const [kernel, setKernel] = useState();
    useEffect(() => {
        if (lite) {
            startLiteServer().then((serviceManager) => {
                setServiceManager(serviceManager);
                setKernelManager(serviceManager.sessions._kernelManager);
            });
        }
        else {
            ensureJupyterAuth(serverSettings).then(isAuth => {
                if (!isAuth) {
                    const loginUrl = getJupyterServerHttpUrl() + '/login?next=' + window.location;
                    console.warn('Redirecting to jupyter login url', loginUrl);
                    window.location.replace(loginUrl);
                }
            });
            const serviceManager = new ServiceManager({ serverSettings });
            setServiceManager(serviceManager);
            const kernelManager = serviceManager.sessions._kernelManager;
            setKernelManager(kernelManager);
            if (createKernel) {
                const kernel = new Kernel({ kernelManager });
                setKernel(kernel);
            }
        }
        setVariant(variant);
    }, [lite, variant]);
    return (_jsx(ReduxProvider, Object.assign({ store: injectableStore }, { children: _jsx(JupyterProvider, Object.assign({ value: {
                lite,
                serverSettings,
                serviceManager,
                kernelManager,
                kernel,
                variant,
                setVariant,
                baseUrl,
                wsUrl,
                injectableStore,
            } }, { children: children }), void 0) }), void 0));
};
//# sourceMappingURL=JupyterContext.js.map